<?php
/**
 * Shopware 4
 * Copyright © shopware AG
 *
 * According to our dual licensing model, this program can be used either
 * under the terms of the GNU Affero General Public License, version 3,
 * or under a proprietary license.
 *
 * The texts of the GNU Affero General Public License with an additional
 * permission and of our proprietary license can be found at and
 * in the LICENSE file you have received along with this program.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * "Shopware" is a registered trademark of shopware AG.
 * The licensing of the program under the AGPLv3 does not imply a
 * trademark license. Therefore any rights, title and interest in
 * our trademarks remain entirely with us.
 */
namespace Shopware\eTracker;

use \Enlight_Controller_Request_RequestHttp as RequestHttp;
use \Enlight_View_Default as View;
use \Shopware\CustomModels\eTracker\Config as Config;

class eTracker
{
	/**
	 * Helper method to get the doctrine entity manager.
	 */
	private function getEntityManager()
	{
		return Shopware()->Models();
	}

	/**
	 * Helper method to actually build the page-name by the given-parameters.
	 *
	 * @param $pageName
	 * @param null $customName
	 * @param null $customId
	 * @return string
	 */
	private function createPageName($pageName, $customName = null, $customId = null)
	{
		if($customName != null) {
			$pageName .= '_' . strtolower(str_replace(' ', '', $customName));
		}

		if($customId != null) {
			$pageName .= '(Id: ' . $customId . ')';
		}
		return $pageName;
	}

	/**
	 * Helper method to build the page-name by a shopware-default.
	 * It has the format controller_action_customName_(Id: Id)
	 * "customName" is for example the article-name or the category-title.
	 *
	 * @param $request \Enlight_Controller_Request_RequestHttp
	 * @param $view \Enlight_View_Default
	 * @return string
	 */
	public function buildPageName(RequestHttp $request, View $view)
	{
		$pageName   = $request->getControllerName() . '_' . $request->getActionName();
		$viewAssign = $view->getAssign();

		switch($request->getControllerName()) {
			case 'blog':
				$pageName = $this->createPageName($pageName, $viewAssign['sArticle']['title'], $viewAssign['sArticle']['id']);
				break;
			case 'detail':
				$pageName = $this->createPageName($pageName, $viewAssign['sArticle']['articleName'], $viewAssign['sArticle']['articleID']);
				break;
			case 'listing':
				$pageName = $this->createPageName($pageName, $viewAssign['sCategoryContent']['name'], $viewAssign['sCategoryContent']['id']);
				break;
			case 'forms':
				$pageName = $this->createPageName($pageName, $viewAssign['sSupport']['name'], $viewAssign['sSupport']['id']);
				break;
			case 'custom':
				$pageName = $this->createPageName($pageName, $viewAssign['sCustomPage']['description'], $viewAssign['sCustomPage']['id']);
				break;
			case 'campaign':
				$pageName = $this->createPageName($pageName, $viewAssign['sBreadCrumb'][0]['name'], $viewAssign['emotionId']);
				break;
            case 'newsletter':
                $pageName = 'newsletter';

                if($request->getParam('subscribeToNewsletter') == '1') {
                    $pageName .= '_success';
                }

				break;
			default:
				break;
		}

		return trim($pageName, '-_/.,;|');
	}

    /**
     * Helper method to get the SearchTerm.
     *
     * @param $request \Enlight_Controller_Request_RequestHttp
     * @param $view \Enlight_View_Default
     * @return string
     */
    public function getSearchTerm(RequestHttp $request, View $view)
    {
        switch($request->getControllerName()) {
            case 'search':
                $searchterm = htmlentities(strip_tags($request->getParam('sSearch')));
                break;
            default:
                $searchterm = '';
                break;
        }

        return trim($searchterm, '-_/.,;|');
    }

    /**
     * Helper method to get the Search Count.
     *
     * @param $request \Enlight_Controller_Request_RequestHttp
     * @param $view \Enlight_View_Default
     * @return int
     */
    public function getSearchCount(RequestHttp $request, View $view)
    {
        switch($request->getControllerName()) {
            case 'search':
                $viewAssign = $view->getAssign();
                $searchCount = intval($viewAssign['sSearchResults']['sArticlesCount']);
                break;
            default:
                $searchCount = 0;
                break;
        }

        return trim($searchCount);
    }

	/**
	 * Helper method to build the page-name from the schema, which the user may set in the config.
	 * It simple replaces the placeholders with actual values.
	 *
	 * @param $request \Enlight_Controller_Request_RequestHttp
	 * @param $view \Enlight_View_Default
	 * @param $config \Shopware\CustomModels\eTracker\Config
	 * @return string
	 */
	public function buildPageNameFromSchema(RequestHttp $request, View $view, Config $config)
	{
		$schema     = $config->getOwnSchema();
		$controller = $request->getControllerName();
		$action     = $request->getActionName();

		list($custom, $id) = $this->getAdditionalInfo($request, $view);

		$schema = str_replace('%controller%', $controller, $schema);
		$schema = str_replace('%action%', $action, $schema);

		if($custom) {
			$schema = str_replace('%custom%', $custom, $schema);
		} else {
			$schema = str_replace('%custom%', '', $schema);
		}

		if($custom) {
			$schema = str_replace('%id%', $id, $schema);
		} else {
			$schema = str_replace('%id%', '', $schema);
		}

		$schema = trim($schema, '-_/.,;|()');

		return $schema;
	}


	/**
	 * Helper method to return if the current frontend-user is logged in already.
	 * The return-value will be given to smarty as an important information for eTracker.
	 *
	 * @return string yes|no
	 */
	public function isLoggedIn()
	{
		if(!Shopware()->Session()->sUserId){
			return 'no';
		}else{
			return 'yes';
		}
	}

	/**
	 * Helper method to get the customergroup of an user.
	 * If the user is not logged in yet, it will return "noGroup"
	 *
	 * @return string
	 */
	public function getCustomerGroup()
	{
		if(!Shopware()->Session()->sUserId){
			return 'noGroup';
		}else{
			$builder = $this->getEntityManager()->createQueryBuilder();
			$builder->select(array('customer', 'customerGroup'))
					->from('Shopware\Models\Customer\Customer', 'customer')
					->innerJoin('customer.group', 'customerGroup')
					->where('customer.id = :id')
					->setParameter('id', Shopware()->Session()->sUserId);

			$customer = $builder->getQuery()->getOneOrNullResult(\Doctrine\ORM\AbstractQuery::HYDRATE_OBJECT);

			return $customer->getGroup()->getName();
		}
	}

	/**
	 * Helper method to get the current interest-level for the current-controller
	 *
	 * @param $request \Enlight_Controller_Request_RequestHttp
	 * @param $config \Shopware\CustomModels\eTracker\Config
	 * @return string
	 */
	public function getInterestLevel(RequestHttp $request, Config $config)
	{
		$controller = $request->getControllerName();
		$interests = $config->getInterests();

		foreach ($interests as $interest)
		{
			$interestController = $interest->getController();
			if(strtolower($interestController->getName()) == $controller)
			{
				return $interest->getLevel();
			}
		}

		return '';
	}

	/**
	 * Helper method to return the sale-state.
	 * It will always return a 1, if the user is on the order-finish page.
	 *
	 * @param $request \Enlight_Controller_Request_RequestHttp
	 * @return string
	 */
	public function getSale(RequestHttp $request)
	{
		return $this->isOnCheckoutFinish($request) ? 1 : '';
	}

	/**
	 * Helper method to check if the user is on the order-finish page.
	 *
	 * @param $request \Enlight_Controller_Request_RequestHttp
	 * @return bool
	 */
	private function isOnCheckoutFinish(RequestHttp $request)
	{
		return $request->getControllerName() == 'checkout' && $action = $request->getActionName() == 'finish';
	}

	/**
	 * Helper method to get the landing-page eTracker id of a landing-page.
	 * This is a new field in the emotion-module.
	 *
	 * @param $request \Enlight_Controller_Request_RequestHttp
	 * @param $view \Enlight_View_Default
	 * @return string
	 */
	public function getLandingPageId(RequestHttp $request, View $view)
	{
		$controller = $request->getControllerName();
		$viewAssign = $view->getAssign();

        if(array_key_exists('emotionId', $viewAssign)) {
            $emotionId = $viewAssign['emotionId'];
        } else {
            $emotionId = $viewAssign['landingPage']['id'];
        }

		if($controller == 'campaign') {
			$emotion = $this->getEntityManager()->find('Shopware\Models\Emotion\Emotion', $emotionId);
			if($attribute = $emotion->getAttribute()){
				return $attribute->getSwagETrackerLandingPageId();
			}
		}

		return '';
	}

	/**
	 * Helper method to get and format the basket.
	 * If the user is on the order-finish page, eTracker wants to have the basket as a specially formatted string.
	 *
	 * @param $request \Enlight_Controller_Request_RequestHttp
	 * @param $view \Enlight_View_Default
	 * @param \Shopware\CustomModels\eTracker\Config $config
	 * @return string
	 */
	public function getBasket(RequestHttp $request, View $view, Config $config)
	{
		$viewAssign = $view->getAssign();
		if($this->isOnCheckoutFinish($request)) {
			$basket = '';

			foreach($viewAssign['sBasket']['content'] as $key=>$basketItem){
				$articleDetail = $this->getEntityManager()->getRepository('Shopware\Models\Article\Detail')->findOneBy(array(
					'number' => $basketItem['ordernumber']
				));
				$category = 'custom';
				if($articleDetail){
					$article = $articleDetail->getArticle();
					$categories = $article->getCategories();
					if(!empty($categories[0])) {
						$category   = implode('/', array_chunk($this->getEntityManager()->getRepository('Shopware\Models\Category\Category')->getPathById($categories[0]->getId()), 4)[0]);
					}
				}

				if($config->getUseNetPrice()) {
					$productPrice = $basketItem['netprice'];
				} else {
					$productPrice = $basketItem['netprice'] * ($basketItem['tax_rate'] / 100 + 1);
				}

				$basket .= $basketItem['ordernumber'] . ',' . $basketItem['articlename'] . ',' . $category . ',' . $basketItem['quantity'] . ',' . $productPrice . ';';
			}

			//Add shippingcosts as own position
			if($config->getUseNetPrice()) {
				$basket .= 'sSHIPPINGCOSTS' . ',' . 'Versandkosten' . ',' . $category . ',' . '1' . ',' . $viewAssign['sBasket']['sShippingcostsNet'] . ',';
			} else {
				$basket .= 'sSHIPPINGCOSTS' . ',' . 'Versandkosten' . ',' . $category . ',' . '1' . ',' . $viewAssign['sShippingcostsWithTax'] . ',';
			}

			return urlencode($basket);
		}

		return '';
	}

	/**
	 * Helper method to check if a user is a new customer.
	 * An user counts as "new" when he has registered on the same day.
	 *
	 * @param $request \Enlight_Controller_Request_RequestHttp
	 * @return string
	 */
	public function isNewCustomer(RequestHttp $request)
	{
		if($this->isOnCheckoutFinish($request))
		{
			$userId = Shopware()->Session()->sUserId;
			$user = $this->getEntityManager()->find('Shopware\Models\Customer\Customer', $userId);
			$firstLoginObject = $user->getFirstLogin();

			//New customer
			return $firstLoginObject->format('Y-m-d') == date('Y-m-d');
		}

		return '';
	}

	/**
	 * Helper method to get the ordernumber of the order on the order-finish page.
	 *
	 * @param $request \Enlight_Controller_Request_RequestHttp
	 * @param $view \Enlight_View_Default
	 * @return string
	 */
	public function getOrderNumber(RequestHttp $request, View $view)
	{
		$viewAssign = $view->getAssign();

		if($this->isOnCheckoutFinish($request)) {
			return $viewAssign['sOrderNumber'];
		}
		return '';
	}

	/**
	 * Helper method to get the amount of an order.
	 * Only a dot may be used for the decimal places and there may be no sign to display a thousand, e.g. 1.000 or 1,000.
	 * It may only be like this: 1000.12
	 *
	 * @param $request \Enlight_Controller_Request_RequestHttp
	 * @param $view \Enlight_View_Default
	 * @param \Shopware\CustomModels\eTracker\Config $config
	 * @return string
	 */
	public function getOrderValue(RequestHttp $request, View $view, Config $config)
	{
		$viewAssign = $view->getAssign();

		if($this->isOnCheckoutFinish($request)) {
			return $this->getPrice($config->getUseNetPrice(), $viewAssign);
		}

		return '';
	}

	/**
	 * Get the net or gross price
	 *
	 * @param $useNetPrice
	 * @param $viewAssign
	 * @return mixed
	 */
	private function getPrice($useNetPrice, $viewAssign)
	{
		if($useNetPrice) {
			return str_replace(',', '.', $viewAssign['sAmountNet']);
		} else {
			if($viewAssign['sAmountWithTax']) {
				return str_replace(',', '.', $viewAssign['sAmountWithTax']);
			} else {
				return str_replace(',', '.', $viewAssign['sAmount']);
			}
		}
	}

	/**
	 * Helper method to get the target of the checkout-pages.
	 * As the user continues through the checkout-process, new targets are reached.
	 * Each target is divided by a "/".
	 *
	 * @param $request \Enlight_Controller_Request_RequestHttp
	 * @param $view \Enlight_View_Default
	 * @return string
	 */
	public function getTarget(RequestHttp $request, View $view)
	{
		$controller = $request->getControllerName();
		$action     = $request->getActionName();
		$viewAssign = $view->getAssign();

		if($controller == 'checkout') {
			if($action == 'cart' && $viewAssign['sBasket']['sAmount']) {
				return 'articleInBasket';
			}

			if($action == 'confirm') {
				return 'articleInBasket/confirmOrder';
			}

			if($action == 'finish') {
				return 'articleInBasket/confirmOrder/orderFinished';
			}
		} elseif($controller == 'register') {
			return 'customerOnRegisterPage';
		}

		return '';
	}

	/**
	 * Helper method to get the affected areas for the current controllers.
	 *
	 * @param $request \Enlight_Controller_Request_RequestHttp
	 * @param $view \Enlight_View_Default
	 * @param $config \Shopware\CustomModels\eTracker\Config
	 * @return string
	 */
	public function getAffectedAreas(RequestHttp $request, View $view, Config $config)
	{
		$divideInto = $config->getDivideAreas();
		$controller = $request->getControllerName();
		$viewAssign = $view->getAssign();
		$area       = '';

		switch($divideInto) {
			//Controller
			case 0:
				$area = $controller;
				break;

			//Categories
			case 1:
				if($controller == 'listing') {
                    $area = implode('/', array_chunk($this->getEntityManager()->getRepository('Shopware\Models\Category\Category')->getPathById($viewAssign['sCategoryContent']['id']), 4)[0]);
				} elseif($controller == 'detail') {
                    $area = implode('/', array_chunk($this->getEntityManager()->getRepository('Shopware\Models\Category\Category')->getPathById($viewAssign['sCategoryInfo']['id']), 4)[0]);
				} else {
					$area = '';
				}
				break;

			//Shops
			case 2:
				$area = Shopware()->Shop()->getName();
				break;

			//Own areas
			case 3:
				$area = $this->handleOwnArea($controller, $viewAssign);
				break;

            // ETracker default
            case 4:
                $controllerArray = array(
                    'detail', 'listing', 'custom', 'forms', 'blog', 'index', 'newsletter', 'sitemap'
                );

                if(in_array($controller, $controllerArray)) {
                    $path = explode('/',$request->getPathInfo());
                    $path = $path[1];

                    if(empty($path)){
                        $path = 'home';
                    }

                    $area = ucfirst($path);
                }
                break;
			default:
				break;
		}

		return urlencode($area);
	}

	/**
	 * Helper method to get the id of the main-shop.
	 *
	 * @return int ID of the main-shop
	 */
	public function getMainShopId()
	{
		if($main = Shopware()->Shop()->getMain()){
			return $main->getId();
		}else{
			return Shopware()->Shop()->getId();
		}
	}

	/**
	 * Helper method to handle the own configured areas.
	 * There may be more than one area for a controller/category/shop.
	 *
	 * @param $currentController
	 * @param $viewAssign
	 * @return string
	 */
	private function handleOwnArea($currentController, $viewAssign)
	{
		$controllerAreas = $this->getEntityManager()->getRepository('Shopware\CustomModels\eTracker\Area')->findBy(array(
			'storeId'       => $this->getMainShopId(),
			'divisionId'    => 1
		));

		$categoryAreas = $this->getEntityManager()->getRepository('Shopware\CustomModels\eTracker\Area')->findBy(array(
			'storeId'       => $this->getMainShopId(),
			'divisionId'    => 2
		));

		$shopAreas = $this->getEntityManager()->getRepository('Shopware\CustomModels\eTracker\Area')->findBy(array(
			'storeId'       => $this->getMainShopId(),
			'divisionId'    => 3
		));

		$result = '';
		foreach($shopAreas as $shopArea) {
			$shops = $shopArea->getShops();

			if(empty($shops)) {
				$result .= $shopArea->getName() . ',';
			} else {
				foreach($shops as $shop) {
					if($shop->getId() == Shopware()->Shop()->getId()) {
						$result .= $shopArea->getName() . ',';
					}
				}
			}
		}

		foreach($controllerAreas as $controllerArea) {
			$controllers = $controllerArea->getControllers();
			if(empty($controllers)) {
				$result .= $controllerArea->getName() . ',';
			} else {
				foreach($controllers as $controller) {
					if(strtolower($controller->getName()) == $currentController) {
						$result .= $controllerArea->getName() . ',';
					}
				}
			}
		}

		foreach($categoryAreas as $categoryArea) {
			if($currentController == 'listing' || $currentController == 'detail') {
				$categories = $categoryArea->getCategories();
				if(empty($categories)) {
					$result .= $categoryArea->getName() . ',';
				} else {
					foreach($categories as $category) {
						if($category->getId() == $viewAssign['sCategoryInfo']['id']) {
							$result .= $categoryArea->getName() . ',';
						}
					}
				}
			}
		}

		return trim($result, ',');
	}

	/**
	 * Helper method to get additional informations for the current controller,
	 * e.g. article-name and article-id.
	 *
	 * @param $request \Enlight_Controller_Request_RequestHttp
	 * @param $view \Enlight_View_Default
	 * @return array|null
	 */
	private function getAdditionalInfo(RequestHttp $request, View $view)
	{
		$viewAssign = $view->getAssign();
		$info       = array();

		switch($request->getControllerName()) {
			case 'blog':
				$info[] = $viewAssign['sArticle']['title'];
				$info[] = $viewAssign['sArticle']['id'];
				break;
			case 'detail':
				$info[] = $viewAssign['sArticle']['articleName'];
				$info[] = $viewAssign['sArticle']['articleID'];
				break;
			case 'listing':
				$info[] = $viewAssign['sCategoryContent']['name'];
				$info[] = $viewAssign['sCategoryContent']['id'];
				break;
			case 'forms':
				$info[] = $viewAssign['sSupport']['name'];
				$info[] = $viewAssign['sSupport']['id'];
				break;
			case 'custom':
				$info[] = $viewAssign['sCustomPage']['description'];
				$info[] = $viewAssign['sCustomPage']['id'];
				break;
			case 'campaign':
				$info[] = $viewAssign['sBreadcrumb'][0]['name'];
				$info[] = $viewAssign['emotionId'];
				break;
			default:
				return null;
		}

		return $info;
	}

	/**
	 * Method to read out necessary information about the needed events.
	 *
	 * @param RequestHttp $request
	 * @param View $view
	 * @return array
	 */
	public function getNecessaryEventInformation(RequestHttp $request, View $view)
	{
		$controller = $request->getControllerName();
		$action     = $request->getActionName();
		$viewAssign = $view->getAssign();
		$events     = array();

		if($controller == 'detail') {
			$sArticle = $viewAssign['sArticle'];
			$events[] = $this->createEvent('viewProduct', $this->getEventArticle($sArticle, $viewAssign), 'send');
            $events[] = $this->createEvent('insertToBasket', $this->getEventArticle($sArticle, $viewAssign), 'attach', 'basketButton');
            $events[] = $this->createEvent('insertToWatchlist', $this->getEventArticle($sArticle, $viewAssign), 'attach', 'watchlistButton');
        } elseif($controller == 'note') {
            foreach ($viewAssign['sNotes'] as $sArticle) {
                $sArticle['categoryID'] = 'Wunsch- / Merkliste';
                $events[]               = $this->createEvent('removeFromWatchlist', $this->getEventArticle($sArticle, $viewAssign), 'attach', 'watchlistButton-' . $sArticle['articleID']);
            }
        } elseif($controller == 'wishlist') {
		    foreach($viewAssign['savedBaskets'] as $savedBasket) {
                foreach ($savedBasket['items'] as $sArticle) {
                    $sArticle               = $sArticle['article'];
                    $sArticle['categoryID'] = 'Wunsch- / Merkliste';
                    $events[]               = $this->createEvent('removeFromWatchlist', $this->getEventArticle($sArticle, $viewAssign), 'attach', 'watchlistButton-' . $sArticle['articleID']);
                }
            }
        } elseif($controller == 'listing') {
            foreach($viewAssign['sArticles'] as $sArticle) {
                $events[] = $this->createEvent('insertToWatchlist', $this->getEventArticle($sArticle, $viewAssign), 'attach', 'watchlistButton-' . $sArticle['articleID']);
            }
        } elseif($controller == 'checkout' && $action == 'finish') {
            if (!empty($viewAssign['sOrderNumber'])) {
                $events[] = $this->createEvent('order', $this->getOrderData($viewAssign), 'send');
            }
		}

		return $events;
	}

	/**
	 * Helper method to create an event-array.
	 *
	 * @param $name
	 * @param $information
	 * @param $type
	 * @param null $attachId
	 * @return array
	 */
	private function createEvent($name, $information, $type, $attachId = null)
	{
		return array('name' => $name, 'information' => $information, 'type' => $type, 'attachId' => $attachId);
	}

	/**
	 * Helper method to filter all the unnecessary information from the article-array.
	 *
	 * @param $sArticle
	 * @param $viewAssign
	 * @return mixed
	 */
	private function getEventArticle($sArticle, $viewAssign)
	{
		$article['id']          = strval($sArticle['articleID']);
		$article['name']        = $sArticle['articleName'];

		if(is_null($sArticle['categoryID'])) {
            if(array_key_exists('sCategoryContent', $viewAssign)) {
                $sArticle['categoryID'] = $viewAssign['sCategoryContent']['id'];
            }
        }

        if(!is_null($sArticle['categoryID'])) {
            if ($sArticle['categoryID'] == 'Wunsch- / Merkliste') {
                $article['category'] = array('Wunsch- / Merkliste');
            } else {
                $categoryPath = implode('/', array_chunk($this->getEntityManager()->getRepository('Shopware\Models\Category\Category')->getPathById($sArticle['categoryID']), 4)[0]);

                if ($categoryPath == 'Root') {
                    $article['category'] = array('Keine Kategorie');
                } else {
                    $article['category'] = explode('/', $categoryPath);
                }
            }
        }

		$sUserData      = Shopware()->Modules()->Admin()->sGetUserData();
		$customerGroup  = Shopware()->Shop()->getCustomerGroup()->getKey();
		if($sUserData['billingaddress']['countryID']) {
			$countryId = $sUserData['billingaddress']['countryID'];
		}

		$customerGroupModel = $this->getEntityManager()->getRepository('Shopware\Models\Customer\Group')->findOneBy(array('key' => $customerGroup));
		$showTax            = $customerGroupModel->getTax();

		$config = Shopware()->Models()->getRepository('Shopware\CustomModels\eTracker\Config')->findOneBy(array(
			'storeId' => $this->getMainShopId()
		));

		if($countryId) {
			$showTax = !$this->getEntityManager()->getRepository('Shopware\Models\Country\Country')->find($countryId)->getTaxFree();
		}

		$normalPrice = str_replace(',', '.', $sArticle['price']);

		if($config->getUseNetPrice()) {
			if(!$showTax) {
				$articlePrice = $normalPrice;
			} else {
				$articlePrice = round(str_replace(',', '.', $sArticle['price']) / ($sArticle['tax'] / 100 + 1), 2);
			}
		} else {
			if(!$showTax) {
				$articlePrice = round(str_replace(',', '.', $sArticle['price']) * ($sArticle['tax'] / 100 + 1), 2);
			} else {
				$articlePrice = $normalPrice;
			}
		}
		$article['price']       = (string) $articlePrice;
		$article['currency']    = Shopware()->Shop()->getCurrency()->getCurrency();
		$article['variants']    = new \stdClass();

		return $article;
	}

	/**
	 * Helper method to create an array with all necessary basket-information.
	 *
	 * @param $viewAssign
	 * @return array
	 */
	private function getOrderData($viewAssign)
	{
		$basket     = $viewAssign['sBasket'];
		$userData   = $viewAssign['sUserData'];
		$products   = array();

		foreach($basket['content'] as $position) {
			if($position['modus'] == 0) {
				$article = Shopware()->Modules()->Articles()->sGetArticleById($position['articleID']);
			} else {
				$article = array(
					'articleID'     => $position['articleID'],
					'articleName'   => $position['articlename'],
					'categoryID'    => 1,
                    'price'         => (string) round($position['price'], 2),
                    'currency'      => Shopware()->Shop()->getCurrency()->getCurrency(),
                    'variants'      => new \stdClass()
				);
			}
			if(!empty($article['articleID'])) {
				$products[] = array('product' => $this->getEventArticle($article, $viewAssign), 'quantity' => (int) $position['quantity']);
			}
		}

        // Versandkosten
        $products[] = array(
            'product'   =>  array(
                'id'            => 'Versandkosten',
                'name'          => 'Versandkosten',
                'category'      => array('Keine Kategorie'),
                'price'         => $basket['sShippingcostsWithTax'],
                'variants'      => new \stdClass(),
                'currency'      => Shopware()->Shop()->getCurrency()->getCurrency()
            ),
            'quantity'  =>  1
        );

		$config = Shopware()->Models()->getRepository('Shopware\CustomModels\eTracker\Config')->findOneBy(array(
			'storeId' => $this->getMainShopId()
		));
		$orderPrice = $this->getPrice($config->getUseNetPrice(), $viewAssign);

		$data = array(
			'orderNumber'           => (string) $viewAssign['sOrderNumber'],
			'status'                => 'sale',
			'orderPrice'            => (string) $orderPrice,
			'currency'              => Shopware()->Shop()->getCurrency()->getCurrency(),
			'basket'                => array(
				'id'        => (string) $this->getEntityManager()->getRepository('Shopware\Models\Order\Order')->findOneBy(array('number' => $viewAssign['sOrderNumber']))->getId(),
				'products'  => $products
			),
			'customerGroup'         => $userData['additional']['user']['customergroup'],
            'deliveryConditions'    => $viewAssign['sDispatch']['name'],
            'paymentConditions'     => $viewAssign['sPayment']['name']
		);

		return $data;
	}
}